use emacs::{defun, Env, IntoLisp, Result, Value};
use std::sync::OnceLock;
use tokio::runtime::{Builder, Runtime};

// Emacs won't load the module without this.
emacs::plugin_is_GPL_compatible!();

// Global static runtime that's lazily initialized
static RUNTIME: OnceLock<Runtime> = OnceLock::new();

// Initialization function for the Emacs module
fn initialize_tokio_runtime() -> &'static Runtime {
    RUNTIME.get_or_init(|| {
        Builder::new_multi_thread()
            .enable_all()
            .worker_threads(4) // Adjust number of worker threads as needed
            .build()
            .expect("Failed to create Tokio runtime")
    })
}

// Register the initialization hook that Emacs will call when it loads the module.
#[emacs::module(separator = "/")]
fn init(env: &Env) -> Result<Value<'_>> {
    let runtime = initialize_tokio_runtime();

    // Spawn a task without blocking
    runtime.spawn(async {
        // Your async code here
        println!("Running an async task");
    });

    env.message("Test loading!")?;

    env.message("Done loading!")
}

// #[defun]
// fn listify_vec(vector: Vector) -> Result<Value> {
//     let mut args = vec![];
//     for e in vector {
//         args.push(e)
//     }
//     vector.0.env.call("list", &args)
// }

// Define a function callable by Lisp code.
#[defun]
fn say_hello(env: &Env, name: String) -> Result<Value<'_>> {
    // env.message(&format!("Helloo    Broooooooo, {}!", name))
    env.call(
        "message",
        [format!("Henlo whatsup, {}!!!!", name)
            .as_str()
            .into_lisp(env)?],
    )?;
    RUNTIME
        .get()
        .ok_or_else(|| anyhow::anyhow!("No runtime"))?
        .spawn(async {
            for _i in 1..5 {
                println!("A");
                std::thread::sleep(std::time::Duration::from_millis(1000));
                println!("B");
            }
        });
    ().into_lisp(env)
}

// #[defun]
// fn open_test_overlay(_env: &Env) -> Result<Value<'_>> {
//     todo!()
// }
